@@ -9,28 +9,64 @@
     :copyright: (c) 2011 by Armin Ronacher and the Django Software Foundation.
     :license: BSD, see LICENSE for more details.
 """
-import base64
-import hashlib
+
+import sys
 import hmac
 import zlib
 import time
-from itertools import izip, imap
+import base64
+import hashlib
+import operator
 from datetime import datetime
 
 
+PY2 = sys.version_info[0] == 2
+if PY2:
+    from itertools import izip
+    text_type = unicode
+    int_to_byte = chr
+else:
+    from functools import reduce
+    izip = zip
+    text_type = str
+    int_to_byte = operator.methodcaller('to_bytes', 1, 'big')
+
+
 try:
-    import simplejson
+    import simplejson as json
 except ImportError:
-    try:
-        from django.utils import simplejson
-    except ImportError:
-        import json as simplejson
+    import json
+
+
+class _CompactJSON(object):
+    """Wrapper around simplejson that strips whitespace.
+    """
+
+    def loads(self, payload):
+        return json.loads(payload)
+
+    def dumps(self, obj):
+        return json.dumps(obj, separators=(',', ':'))
+
+
+compact_json = _CompactJSON()
 
 
 # 2011/01/01 in UTC
 EPOCH = 1293840000
 
 
+def want_bytes(s, encoding='utf-8', errors='strict'):
+    if isinstance(s, text_type):
+        s = s.encode(encoding, errors=errors)
+    return s
+
+
+def is_text_serializer(serializer):
+    """Checks weather a serializer generates text or binary."""
+    return isinstance(serializer.dumps({}), text_type)
+
+
 def constant_time_compare(val1, val2):
     """Returns True if the two strings are equal, False otherwise.
 
@@ -47,8 +83,8 @@ def constant_time_compare(val1, val2):
     else:
         result = 1
         left = val2
-    for x, y in izip(left, val2):
-        result |= ord(x) ^ ord(y)
+    for x, y in izip(bytearray(left), bytearray(val2)):
+        result |= x ^ y
     return result == 0
 
 
@@ -65,10 +101,12 @@ class BadData(Exception):
         self.message = message
 
     def __str__(self):
-        return self.message
+        return text_type(self.message)
 
-    def __unicode__(self):
-        return self.message.decode('utf-8')
+    if PY2:
+        __unicode__ = __str__
+        def __str__(self):
+            return self.__unicode__().encode('utf-8')
 
 
 class BadPayload(BadData):
@@ -127,30 +165,34 @@ class SignatureExpired(BadTimeSignature):
 
 
 def base64_encode(string):
-    """base64 encodes a single string.  The resulting string is safe for
-    putting into URLs.
+    """base64 encodes a single bytestring (and is tolerant to getting
+    called with a unicode string).
+    The resulting bytestring is safe for putting into URLs.
     """
-    return base64.urlsafe_b64encode(string).strip('=')
+    string = want_bytes(string)
+    return base64.urlsafe_b64encode(string).strip(b'=')
 
 
 def base64_decode(string):
-    """base64 decodes a single string."""
-    if isinstance(string, unicode):
-        string = string.encode('ascii', 'ignore')
-    return base64.urlsafe_b64decode(string + '=' * (-len(string) % 4))
+    """base64 decodes a single bytestring (and is tolerant to getting
+    called with a unicode string).
+    The result is also a bytestring.
+    """
+    string = want_bytes(string, encoding='ascii', errors='ignore')
+    return base64.urlsafe_b64decode(string + b'=' * (-len(string) % 4))
 
 
 def int_to_bytes(num):
     assert num >= 0
     rv = []
     while num:
-        rv.append(chr(num & 0xff))
+        rv.append(int_to_byte(num & 0xff))
         num >>= 8
-    return ''.join(reversed(rv))
+    return b''.join(reversed(rv))
 
 
-def bytes_to_int(bytes):
-    return reduce(lambda a, b: a << 8 | b, imap(ord, bytes), 0)
+def bytes_to_int(bytestr):
+    return reduce(lambda a, b: a << 8 | b, bytearray(bytestr), 0)
 
 
 class SigningAlgorithm(object):
@@ -169,7 +211,7 @@ class NoneAlgorithm(SigningAlgorithm):
     """
 
     def get_signature(self, key, value):
-        return ''
+        return b''
 
 
 class HMACAlgorithm(SigningAlgorithm):
@@ -190,8 +232,8 @@ class HMACAlgorithm(SigningAlgorithm):
 
 
 class Signer(object):
-    """This class can sign a string and unsign it and validate the
-    signature provided.
+    """This class can sign bytes and unsign it and validate the signature
+    provided.
 
     Salt can be used to namespace the hash, so that a signed string is only
     valid for a given namespace.  Leaving this at the default value or re-using
@@ -226,9 +268,9 @@ class Signer(object):
 
     def __init__(self, secret_key, salt=None, sep='.', key_derivation=None,
                  digest_method=None, algorithm=None):
-        self.secret_key = secret_key
+        self.secret_key = want_bytes(secret_key)
         self.sep = sep
-        self.salt = salt or 'itsdangerous.Signer'
+        self.salt = 'itsdangerous.Signer' if salt is None else salt
         if key_derivation is None:
             key_derivation = self.default_key_derivation
         self.key_derivation = key_derivation
@@ -246,14 +288,15 @@ class Signer(object):
         to be used as a security method to make a complex key out of a short
         password.  Instead you should use large random secret keys.
         """
+        salt = want_bytes(self.salt)
         if self.key_derivation == 'concat':
-            return self.digest_method(self.salt + self.secret_key).digest()
+            return self.digest_method(salt + self.secret_key).digest()
         elif self.key_derivation == 'django-concat':
-            return self.digest_method(self.salt + 'signer' +
+            return self.digest_method(salt + b'signer' +
                 self.secret_key).digest()
         elif self.key_derivation == 'hmac':
             mac = hmac.new(self.secret_key, digestmod=self.digest_method)
-            mac.update(self.salt)
+            mac.update(salt)
             return mac.digest()
         elif self.key_derivation == 'none':
             return self.secret_key
@@ -262,26 +305,25 @@ class Signer(object):
 
     def get_signature(self, value):
         """Returns the signature for the given value"""
+        value = want_bytes(value)
         key = self.derive_key()
         sig = self.algorithm.get_signature(key, value)
         return base64_encode(sig)
 
     def sign(self, value):
         """Signs the given string."""
-        if isinstance(value, unicode):
-            value = value.encode('utf-8')
-        return '%s%s%s' % (value, self.sep, self.get_signature(value))
+        return value + want_bytes(self.sep) + self.get_signature(value)
 
     def unsign(self, signed_value):
         """Unsigns the given string."""
-        if isinstance(signed_value, unicode):
-            signed_value = signed_value.encode('utf-8')
-        if self.sep not in signed_value:
-            raise BadSignature('No "%s" found in value' % self.sep)
-        value, sig = signed_value.rsplit(self.sep, 1)
+        signed_value = want_bytes(signed_value)
+        sep = want_bytes(self.sep)
+        if sep not in signed_value:
+            raise BadSignature('No %r found in value' % self.sep)
+        value, sig = signed_value.rsplit(sep, 1)
         if constant_time_compare(sig, self.get_signature(value)):
             return value
-        raise BadSignature('Signature "%s" does not match' % sig,
+        raise BadSignature('Signature %r does not match' % sig,
                            payload=value)
 
     def validate(self, signed_value):
@@ -316,11 +358,11 @@ class TimestampSigner(Signer):
 
     def sign(self, value):
         """Signs the given string and also attaches a time information."""
+        value = want_bytes(value)
         timestamp = base64_encode(int_to_bytes(self.get_timestamp()))
-        value = '%s%s%s' % (value, self.sep, timestamp)
-        if isinstance(value, unicode):
-            value = value.encode('utf-8')
-        return '%s%s%s' % (value, self.sep, self.get_signature(value))
+        sep = want_bytes(self.sep)
+        value = value + sep + timestamp
+        return value + sep + self.get_signature(value)
 
     def unsign(self, value, max_age=None, return_timestamp=False):
         """Works like the regular :meth:`~Signer.unsign` but can also
@@ -332,21 +374,22 @@ class TimestampSigner(Signer):
         try:
             result = Signer.unsign(self, value)
             sig_error = None
-        except BadSignature, e:
+        except BadSignature as e:
             sig_error = e
-            result = e.payload or ''
+            result = e.payload or b''
+        sep = want_bytes(self.sep)
 
         # If there is no timestamp in the result there is something
         # seriously wrong.  In case there was a signature error, we raise
         # that one directly, otherwise we have a weird situation in which
         # we shouldn't have come except someone uses a time-based serializer
         # on non-timestamp data, so catch that.
-        if not self.sep in result:
+        if not sep in result:
             if sig_error:
                 raise sig_error
             raise BadTimeSignature('timestamp missing', payload=result)
 
-        value, timestamp = result.rsplit(self.sep, 1)
+        value, timestamp = result.rsplit(sep, 1)
         try:
             timestamp = bytes_to_int(base64_decode(timestamp))
         except Exception:
@@ -355,13 +398,13 @@ class TimestampSigner(Signer):
         # Signature is *not* okay.  Raise a proper error now that we have
         # split the value and the timestamp.
         if sig_error is not None:
-            raise BadTimeSignature(unicode(sig_error), payload=value,
+            raise BadTimeSignature(text_type(sig_error), payload=value,
                                    date_signed=timestamp)
 
         # Signature was okay but the timestamp is actually not there or
         # malformed.  Should not happen, but well.  We handle it nonetheless
         if timestamp is None:
-            raise BadTimeSignature(u'Malformed timestamp', payload=value)
+            raise BadTimeSignature('Malformed timestamp', payload=value)
 
         # Check timestamp is not older than max_age
         if max_age is not None:
@@ -388,18 +431,19 @@ class TimestampSigner(Signer):
 
 class Serializer(object):
     """This class provides a serialization interface on top of the
-    signer.  It provides a similar API to json/pickle/simplejson and
-    other modules but is slightly differently structured internally.
-    If you want to change the underlying implementation for parsing and
-    loading you have to override the :meth:`load_payload` and
-    :meth:`dump_payload` functions.
+    signer.  It provides a similar API to json/pickle and other modules but is
+    slightly differently structured internally.  If you want to change the
+    underlying implementation for parsing and loading you have to override the
+    :meth:`load_payload` and :meth:`dump_payload` functions.
 
-    This implementation uses simplejson for dumping and loading.
+    This implementation uses simplejson if available for dumping and loading
+    and will fall back to the standard library's json module if it's not
+    available.
 
-    Starting with 0.14 you do not need to subclass this class in order
-    to switch out or customer the :class:`Signer`.  You can instead
-    also pass a different class to the constructor as well as
-    keyword arguments as dictionary that should be forwarded::
+    Starting with 0.14 you do not need to subclass this class in order to
+    switch out or customer the :class:`Signer`.  You can instead also pass a
+    different class to the constructor as well as keyword arguments as
+    dictionary that should be forwarded::
 
         s = Serializer(signer_kwargs={'key_derivation': 'hmac'})
 
@@ -410,7 +454,7 @@ class Serializer(object):
 
     #: If a serializer module or class is not passed to the constructor
     #: this one is picked up.  This currently defaults to :mod:`json`.
-    default_serializer = simplejson
+    default_serializer = json
 
     #: The default :class:`Signer` class that is being used by this
     #: serializer.
@@ -418,40 +462,45 @@ class Serializer(object):
     #: .. versionadded:: 0.14
     default_signer = Signer
 
-    def __init__(self, secret_key, salt='itsdangerous', serializer=None,
+    def __init__(self, secret_key, salt=b'itsdangerous', serializer=None,
                  signer=None, signer_kwargs=None):
-        self.secret_key = secret_key
-        self.salt = salt
+        self.secret_key = want_bytes(secret_key)
+        self.salt = want_bytes(salt)
         if serializer is None:
             serializer = self.default_serializer
         self.serializer = serializer
+        self.is_text_serializer = is_text_serializer(serializer)
         if signer is None:
             signer = self.default_signer
         self.signer = signer
         self.signer_kwargs = signer_kwargs or {}
 
     def load_payload(self, payload, serializer=None):
-        """Loads the encoded object.  This implementation uses simplejson.
-        This function raises :class:`BadPayload` if the payload is not
-        valid.  The `serializer` parameter can be used to override the
-        serializer stored on the class.
+        """Loads the encoded object.  This function raises :class:`BadPayload`
+        if the payload is not valid.  The `serializer` parameter can be used to
+        override the serializer stored on the class.  The encoded payload is
+        always byte based.
         """
         if serializer is None:
             serializer = self.serializer
+            is_text = self.is_text_serializer
+        else:
+            is_text = is_text_serializer(serializer)
         try:
-            if isinstance(payload, unicode):
-                payload = payload.encode('utf-8')
+            if is_text:
+                payload = payload.decode('utf-8')
             return serializer.loads(payload)
-        except Exception, e:
-            raise BadPayload(u'Could not load the payload because an '
-                u'exception ocurred on unserializing the data',
+        except Exception as e:
+            raise BadPayload('Could not load the payload because an '
+                'exception ocurred on unserializing the data',
                 original_error=e)
 
     def dump_payload(self, obj):
-        """Dumps the encoded object into a bytestring.  This implementation
-        uses simplejson.
+        """Dumps the encoded object.  The return value is always a
+        bytestring.  If the internal serializer is text based the value
+        will automatically be encoded to utf-8.
         """
-        return self.serializer.dumps(obj)
+        return want_bytes(self.serializer.dumps(obj))
 
     def make_signer(self, salt=None):
         """A method that creates a new instance of the signer to be used.
@@ -462,22 +511,29 @@ class Serializer(object):
         return self.signer(self.secret_key, salt=salt, **self.signer_kwargs)
 
     def dumps(self, obj, salt=None):
-        """Returns URL-safe, signed base64 compressed JSON string.
-
-        If compress is True (the default) checks if compressing using zlib can
-        save some space. Prepends a '.' to signify compression. This is included
-        in the signature, to protect against zip bombs.
+        """Returns a signed string serialized with the internal serializer.
+        The return value can be either a byte or unicode string depending
+        on the format of the internal serializer.
         """
-        return self.make_signer(salt).sign(self.dump_payload(obj))
+        payload = self.dump_payload(obj)
+        if isinstance(payload, text_type):
+            payload = payload.encode('utf-8')
+        rv = self.make_signer(salt).sign(payload)
+        if self.is_text_serializer:
+            rv = rv.decode('utf-8')
+        return rv
 
     def dump(self, obj, f, salt=None):
-        """Like :meth:`dumps` but dumps into a file."""
+        """Like :meth:`dumps` but dumps into a file.  The file handle has
+        to be compatible with what the internal serializer expects.
+        """
         f.write(self.dumps(obj, salt))
 
     def loads(self, s, salt=None):
         """Reverse of :meth:`dumps`, raises :exc:`BadSignature` if the
         signature validation fails.
         """
+        s = want_bytes(s)
         return self.load_payload(self.make_signer(salt).unsign(s))
 
     def load(self, f, salt=None):
@@ -506,7 +562,7 @@ class Serializer(object):
         """
         try:
             return True, self.loads(s, salt=salt, **(load_kwargs or {}))
-        except BadSignature, e:
+        except BadSignature as e:
             if e.payload is None:
                 return False, None
             try:
@@ -566,31 +622,32 @@ class JSONWebSignatureSerializer(Serializer):
     #: The default algorithm to use for signature generation
     default_algorithm = 'HS256'
 
-    def __init__(self, secret_key, salt=None, signer_kwargs=None, algorithm_name=None):
-        self.secret_key = secret_key
-        self.salt = salt
-        self.serializer = compact_json
-        self.signer = Signer
-        self.signer_kwargs = signer_kwargs or {}
+    default_serializer = compact_json
+
+    def __init__(self, secret_key, salt=None, serializer=None,
+                 signer=None, signer_kwargs=None, algorithm_name=None):
+        Serializer.__init__(self, secret_key, salt, serializer,
+                            signer, signer_kwargs)
         if algorithm_name is None:
             algorithm_name = self.default_algorithm
         self.algorithm_name = algorithm_name
         self.algorithm = self.make_algorithm(algorithm_name)
 
     def load_payload(self, payload, return_header=False):
-        if '.' not in payload:
+        payload = want_bytes(payload)
+        if b'.' not in payload:
             raise BadPayload('No "." found in value')
-        base64d_header, base64d_payload = payload.split('.', 1)
+        base64d_header, base64d_payload = payload.split(b'.', 1)
         try:
             json_header = base64_decode(base64d_header)
             json_payload = base64_decode(base64d_payload)
-        except Exception, e:
-            raise BadPayload(u'Could not base64 decode the payload because of '
-                u'an exception', original_error=e)
+        except Exception as e:
+            raise BadPayload('Could not base64 decode the payload because of '
+                'an exception', original_error=e)
         header = Serializer.load_payload(self, json_header,
-            serializer=simplejson)
+            serializer=json)
         if not isinstance(header, dict):
-            raise BadPayload(u'Header payload is not a JSON object')
+            raise BadPayload('Header payload is not a JSON object')
         payload = Serializer.load_payload(self, json_payload)
         if return_header:
             return payload, header
@@ -599,17 +656,17 @@ class JSONWebSignatureSerializer(Serializer):
     def dump_payload(self, header, obj):
         base64d_header = base64_encode(self.serializer.dumps(header))
         base64d_payload = base64_encode(self.serializer.dumps(obj))
-        return '%s.%s' % (base64d_header, base64d_payload)
+        return base64d_header + b'.' + base64d_payload
 
     def make_algorithm(self, algorithm_name):
         try:
             return self.jws_algorithms[algorithm_name]
         except KeyError:
-            raise NotImplementedError("Algorithm not supported")
+            raise NotImplementedError('Algorithm not supported')
 
     def make_signer(self, salt=None, algorithm=None):
         if salt is None:
-            salt is self.salt
+            salt = self.salt
         key_derivation = 'none' if salt is None else None
         if algorithm is None:
             algorithm = self.algorithm
@@ -635,7 +692,7 @@ class JSONWebSignatureSerializer(Serializer):
         return a tuple of payload and header.
         """
         payload, header = self.load_payload(
-            self.make_signer(salt, self.algorithm).unsign(s),
+            self.make_signer(salt, self.algorithm).unsign(want_bytes(s)),
             return_header=True)
         if header.get('alg') != self.algorithm_name:
             raise BadSignature('Algorithm mismatch')
@@ -656,20 +713,20 @@ class URLSafeSerializerMixin(object):
 
     def load_payload(self, payload):
         decompress = False
-        if payload[0] == '.':
+        if payload[0] == b'.':
             payload = payload[1:]
             decompress = True
         try:
             json = base64_decode(payload)
-        except Exception, e:
-            raise BadPayload(u'Could not base64 decode the payload because of '
-                u'an exception', original_error=e)
+        except Exception as e:
+            raise BadPayload('Could not base64 decode the payload because of '
+                'an exception', original_error=e)
         if decompress:
             try:
                 json = zlib.decompress(json)
-            except Exception, e:
-                raise BadPayload(u'Could not zlib decompress the payload before '
-                    u'decoding the payload', original_error=e)
+            except Exception as e:
+                raise BadPayload('Could not zlib decompress the payload before '
+                    'decoding the payload', original_error=e)
         return super(URLSafeSerializerMixin, self).load_payload(json)
 
     def dump_payload(self, obj):
@@ -681,24 +738,10 @@ class URLSafeSerializerMixin(object):
             is_compressed = True
         base64d = base64_encode(json)
         if is_compressed:
-            base64d = '.' + base64d
+            base64d = b'.' + base64d
         return base64d
 
 
-class _CompactJSON(object):
-    """Wrapper around simplejson that strips whitespace.
-    """
-
-    def loads(self, payload):
-        return simplejson.loads(payload)
-
-    def dumps(self, obj):
-        return simplejson.dumps(obj, separators=(',', ':'))
-
-
-compact_json = _CompactJSON()
-
-
 class URLSafeSerializer(URLSafeSerializerMixin, Serializer):
     """Works like :class:`Serializer` but dumps and loads into a URL
     safe string consisting of the upper and lowercase character of the
